// package camickr;
/************************************************************************
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.List;
import java.lang.StringBuffer;

/**
 * <code>Listing</code> class categorizes and lists the key/value
 * pairs contained within the <code>javax.swing.UIDefaults</code>
 * collection.
 *
 * @author Dr. Lazlo Jamf
 * @see    <a href="http://forum.java.sun.com/thread.jspa?threadID=555184&forumID=57">Dr. Laszlo Jamf's original post</a>
 * @see    <code>javax.swing.UIDefaults</code>
 *
 */
public class Listing
{
    public static void main(String[] args) throws Exception
    {
        JFrame.setDefaultLookAndFeelDecorated(true);
        System.setProperty("sun.awt.noerasebackground", "true");
        JFrame f = new JFrame("Listing");
        JPanel view = new JPanel(new GridBagLayout());
        Container controls = new JToolBar();
        controls.add(Box.createGlue());
        controls.add(getTypesCombo(view));
        controls.add(Box.createGlue());
        Container cp = f.getContentPane();
        cp.add(new JScrollPane(view));
        cp.add(controls, BorderLayout.NORTH);
        f.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        f.setSize(600, 600);
        f.setLocationRelativeTo(null);
        f.setVisible(true);
    }

    /**
     * Statically creates a <code>javax.swing.JLabel</code> object and returns this object as
     * <code>javax.swing.JComponent</code>.
     *
     * @param key   <code>java.lang.Object</code> of the <code>key</code> value of this collection
     * @param value <code>java.lang.Object</code> of the <code>value</code> of this collection
     */
    static class ComponentMaker
    {
        public JComponent make(Object key, Object value)
        {
            return new JLabel(value == null ? "(null)" : value.toString());
        }
    }

    /**
     * Component factory creates an array of <code>ComponentMaker</code> objects of all the categorized UI types.
     * i.e. (Boolean, Border, Color, Font, , Number, String, and The Rest).
     */
    static ComponentMaker[] makers =
    {
        new ComponentMaker(),   //Boolean
        new ComponentMaker()
        {   //Border
            public JComponent make(Object key, Object value)
            {
                JLabel lbl = new JLabel("        ");
                lbl.setBorder(new BorderAdapter((Border)value, key.toString()));
                return lbl;
            }
        },
        new ComponentMaker()
        {   //Color
            public JComponent make(Object key, Object value)
            {
                return new JLabel(new ColorIcon((Color)value));
            }
        },
        new ComponentMaker()
        {   //Font
            public JComponent make(Object key, Object value)
            {
                Font font = (Font)value;
                JLabel lbl = new JLabel(key+": "+font.getFontName()+", size="+font.getSize2D());
                lbl.setFont(font);
                return lbl;
            }
        },
        new ComponentMaker()
        {   //Icon
            public JComponent make(Object key, Object value)
            {
                return new JLabel(new IconAdapter((Icon)value, key.toString()));
            }
        },
        new ComponentMaker(),   //Number
        new ComponentMaker(),   //String
        new ComponentMaker()    //The Rest
    };


    /**
     * Returns the <code>types javax.swing.JComboBox</code> listing of all the LAF properties and resources.
     * Attachs <code>java.awt.event.ActionListener</code> to the <code>cboTypes javax.swing.JComboBox</code>.
     *
     * @param  view     <code>javax.swing.JPanel</code> of this view.
     * @return cboTypes <code>javax.swing.JComboBox</code> of all UIDefaults properties and resources.
     */
    @SuppressWarnings(value = "unchecked") // suppresses the raw List warnings
    static JComboBox getTypesCombo(final JPanel view)
    {
        JComboBox cboTypes = new JComboBox(
            new String[]{"Boolean","Border","Color","Font","Icon", "Number", "String", "The Rest"});
        cboTypes.addActionListener(new ActionListener()  // anonymous class
        {
            Class[] types = {Boolean.class, Border.class, Color.class, Font.class, Icon.class, Number.class, String.class};

            public void actionPerformed(ActionEvent evt)
            {
                int idx = ((JComboBox) evt.getSource()).getSelectedIndex();
                if (idx != -1)
                {
                    view.removeAll();
                    List keys = (idx < types.length) ? findKeys(types[idx]) : findRestKeys(types);//
                    GridBagConstraints gbc = new GridBagConstraints();
                    gbc.anchor = GridBagConstraints.FIRST_LINE_START;
                    gbc.gridy = 0;
                    for(int i=0, ub=keys.size(); i<ub; ++i) {
                        if (i ==ub-1)
                            gbc.weighty = 1;
                        Object key = keys.get(i);
                        gbc.weightx = 0;
                        view.add(new JLabel(key.toString()), gbc);
                        gbc.weightx = 1;
                        view.add(makers[idx].make(key, UIManager.get(key)), gbc);
                        gbc.gridy++;
                    }
                    view.revalidate();
                    Component parent = view.getParent();
                    if (parent instanceof JViewport)
                    {
                        ((JViewport) parent).setViewPosition(new Point(0,0));
                    }
                }
            }
        }); // End anonymous
        cboTypes.setSelectedIndex(2);
        return cboTypes;
    }

    /**
     * Finds the keys in the javax.swing.UIDefaults collection for a
     * particular class and sorts the keys using the
     * <code>java.util.Collections#sort</code> method.
     *
     * @param  cls  <code>java.lang.Class</code> of one of the defined types.
     * @return keys <code>java.util.List</code> of this class' keys.
     */
    @SuppressWarnings(value = "unchecked")
    static List findKeys(Class cls)
    {
        List keys = new ArrayList();
        UIDefaults defs = UIManager.getLookAndFeel().getDefaults();
        for(Iterator i=defs.keySet().iterator(); i.hasNext(); )
        {
            Object key = i.next();
            if(cls.isInstance(UIManager.get(key)))
            {
                keys.add(key);
            }
        }
        Collections.sort(keys);
        return keys;
    }

    /**
     * Finds the rest of the keys in the javax.swing.UIDefaults collection
     * and sorts the keys using <code>Collections#sort</code>.
     *
     * @param  cls  <code>java.lang.Class</code> array of defined class types.
     * @return keys <code>java.util.List</code> of class keys.
     */
    @SuppressWarnings(value = "unchecked")
    static List findRestKeys(Class[] cls)
    {
        List keys = new ArrayList();
        StringBuffer sbuf;
        UIDefaults defs = UIManager.getLookAndFeelDefaults();
        for(Iterator<Object> i = defs.keySet().iterator(); i.hasNext();)
        {
            Object key = (Object) i.next();
            Object value = (Object) UIManager.get(key);
            int j;
            // Modified: Brent Allen Parrish 5/10/2008
            // ADDED: A test to check for the instance of
            // a java.lang.String. The Collections#sort method will throw
            // a ClassCastException if an attempt to compare an
            // immutable object(String) and a mutable object(StringBuffer)
            // is made, which was the case here - comparing a
            // java.lang.StringBuffer to a java.lang.String.
            if(key instanceof String)
            {
                sbuf = new StringBuffer(key.toString());
                key = (Object)sbuf;
            }
            // End mod
            for(j = 0; j < cls.length; ++j)
            {
                if(cls[j].isInstance(value))
                {
                    break;
                }
            }
            if (j == cls.length)
            {
                // Modified: Brent Allen Parrish 5/10/2008
                // ADDED: see above comment.
                if(key instanceof StringBuffer)
                {
                    keys.add(key.toString());
                }
                else
                {
                    keys.add(key);
                }
                // End mod
            }
        }
         // Modified: Brent Allen Parrish 5/10/2008
         // ADDED: try/catch block for ClassCastException
         // thrown by the Collections#sort method when
         // immutable and mutable object comparisons are
         // attempted. The catch block will return the key
         // set unsorted if an exception is thrown.
        try
        {
            Collections.sort(keys);
        }
        catch(ClassCastException cce)
        {
           System.out.println(cce.getMessage() + "\n");
           cce.printStackTrace();
           return keys;
        }
        // End mod
        return keys;
    }

} // End class

/**
 * <code>ColorIcon</code> class handles <code>ColorIcon</code> rendering and implements the
 * <code>javax.swing.Icon interface</code>.
 *
 */
class ColorIcon implements Icon
{

    private Color color;

    /**
     * Sets this member's <code>java.awt.Color</code> reference.
     *
     * @param color <code>java.awt.Color</code> to this instance.
     */
    public ColorIcon(Color color)
    {
        this.color = color;
    }

    /**
     * Returns the icon width.
     *
     * @return <code>int</code> value representing the icon width in pixels.
     */
    public int getIconWidth()
    {
        return 20;
    }

    /**
     * Returns the icon height.
     *
     * @return <code>int</code> value representing the icon height in pixels.
     */
    public int getIconHeight()
    {
        return 16;
    }

    /**
     * Renders the icon.
     *
     * @param c <code>java.awt.Component</code> to paint.
     * @param g <code>java.awt.Graphics</code> object.
     * @param x <code>int</code> value for <code>x</code> axis coordinate.
     * @param y <code>int</code> value for <code>y</code> axis coordinate.
     */
    public void paintIcon(Component c, Graphics g, int x, int y)
    {
        if (color != null)
        {
            g.setColor(color);
            g.fillRect(x, y, getIconWidth(), getIconHeight());
            g.setColor(Color.BLACK);
            g.drawRect(x, y, getIconWidth()-1, getIconHeight()-1);
        }
    }
}

/**
 * <code>IconAdapter</code> class handles this instance's painting of <code>javax.swing.ImageIcon</code>
 * and <code>javax.swing.Icon interface</object> objects.
 *
 * @param icon <code>javax.swing.Icon interface</code> object for this instance.
 * @param key  <code>java.lang.String</code> of this instance's
 *             <code>javax.swing.UIDefaults</code> key attribute.
 * @author     Dr. Lazlo Jamf <Sun Java Developer's Forum user: DrLazloJamf>
 */
class IconAdapter implements Icon
{
    private Icon icon;
    private Component component;

    /**
     * Class constructor sets an instance of the <code>javax.swing.Icon</code> interface
     * and a <code>java.lang.String</code> containing the qualified javax.swing.JComponent
     * class name.
     *
     * @param icon <code>javax.swing.Icon interface</code> instance.
     * @param key  <code>java.lang.String</code> containing qualified <code>javax.swing.JComponent</code>
     *             class name.
     * @see        <code>javax.swing.Icon interface</code>
     */
    public IconAdapter(Icon icon, String key)
    {
        this.icon = icon;
        try
        {
            int dotIndex = key.lastIndexOf('.');
            if (dotIndex != -1)
            {
                String classname = "javax.swing.J" + key.substring(0, dotIndex);
                component = (Component) Class.forName(classname).newInstance();
            }
        }
        catch (Exception e)
        {
        }
    }

    /**
     * Accessor for icon pixel width.
     *
     * @return <code>int</code> value representing icon pixel width.
     */
    public int getIconWidth()
    {
        return icon.getIconWidth();
    }

    /**
     * Accessor for icon pixel height.
     *
     * @return <code>int</code> value representing icon pixel height.
     */
    public int getIconHeight()
    {
        return icon.getIconHeight();
    }

    /**
     * Paints icon at this x, y location.
     *
     * @param c <code>java.awt.Component</code> reference of the component to paint.
     * @param g <code>java.awt.Graphics</code> object.
     * @param x <code>int</code> value of the x axis coordinate.
     * @param y <code>int</code> value of the y axis coordinate.
     */
    public void paintIcon(Component c, Graphics g, int x, int y)
    {
        try
        {
            if (component != null)
            {
                c = component;
            }
            icon.paintIcon(c, g, x, y);
        }
        catch(Exception e)
        {
        }
    }
}

/**
 * <code>BorderAdapter</code> class handles this instance's displaying of
 * <code>javax.swing.border.Border</code> objects.
 *
 * @param icon <code>javax.swing.Border interface</code> instance.
 * @param key  <code>java.lang.String</code> of this instance's
 *             <code>javax.swing.UIDefaults</code> key attribute.
 * @author     Dr. Lazlo Jamf <Sun Java Developer's Forum user: DrLazloJamf>
 */
class BorderAdapter implements Border
{
    private Border border;
    private Component component;

     /**
      * Class constructor sets an instance of the <code>javax.swing.Border</code> interface
      * and a <code>java.lang.String</code> containing the qualified javax.swing.JComponent
      * class name.
      *
      * @param border <code>javax.swing.Border interface</code> instance.
      * @param key    <code>java.lang.String</code> containing qualified <code>javax.swing.JComponent</code>
      *               class name.
      * @see          <code>javax.swing.Border interface</code>
      */
    public BorderAdapter(Border border, String key)
    {
        this.border = border;
        try
        {
            int dotIndex = key.lastIndexOf('.');
            if (dotIndex != -1)
            {
                String classname = "javax.swing.J" + key.substring(0, dotIndex);
                component = (Component) Class.forName(classname).newInstance();
            }
        }
        catch (Exception e)
        {
        }
    }

    /**
     * Returns the <code>boolean</code> value of the border#isBorderOpaque method.
     *
     * @return <code>boolean</code> value for this instance's border#isBorderOpaque method.
     */
    public boolean isBorderOpaque()
    {
        return border.isBorderOpaque();
    }

    /**
     * Accessor for border's <code>java.awt.Insets</code>.
     *
     * @return <code>java.awt.Insets</code> of this border's insets.
     */
    public Insets getBorderInsets(Component c)
    {
        try
        {
            if (component != null)
            {
                c = component;
            }
            return border.getBorderInsets(c);
        } catch(Exception e)
        {
            return new Insets(0,0,0,0);
        }
    }

    /**
     * Paints border at this x, y location.
     *
     * @param c <code>java.awt.Component</code> reference to the component to paint.
     * @param g <code>java.awt.Graphics</code> object.
     * @param x <code>int</code> value of the x axis coordinate.
     * @param y <code>int</code> value of the y axis coordinate.
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)
    {
        try
        {
            if (component != null)
            {
                c = component;
            }
            border.paintBorder(c, g, x, y, width, height);
        }
        catch(Exception e)
        {
        }
    }
}



